import sys
import uuid

import pytest


@pytest.mark.skipif(sys.platform != "darwin", reason="macOS only")
def test_keyring_non_ascii_repo():
    from vorta.keyring.darwin import VortaDarwinKeyring

    UNICODE_PW = 'password'
    REPO = 'vorta-test-repo-한글'

    keyring = VortaDarwinKeyring()
    keyring.set_password('vorta-repo', REPO, UNICODE_PW)
    assert keyring.get_password("vorta-repo", REPO) == UNICODE_PW



from test_constants import TEST_SOURCE_DIR

from vorta.borg.create import BorgCreateJob
from vorta.store.models import BackupProfileModel, SourceFileModel


def test_create_paths_from_command():
    default_profile = BackupProfileModel.get()
    default_profile.new_archive_name = 'a1'

    default_profile.repo.create_backup_cmd = '--one-file-system'
    result = BorgCreateJob.prepare(default_profile)

    assert 'cmd' in result
    assert result['cmd'] == [
        'borg',
        'create',
        '--list',
        '--progress',
        '--info',
        '--log-json',
        '--json',
        '--filter=AM',
        '-C',
        'lz4',
        '--one-file-system',
        'i0fi93@i593.repo.borgbase.com:repo::a1',
        TEST_SOURCE_DIR,
    ]

    default_profile.repo.create_backup_cmd = f'--paths-from-command -- echo {TEST_SOURCE_DIR}'
    SourceFileModel.delete().execute()

    result = BorgCreateJob.prepare(default_profile)

    assert 'cmd' in result
    assert result['cmd'] == [
        'borg',
        'create',
        '--list',
        '--progress',
        '--info',
        '--log-json',
        '--json',
        '--filter=AM',
        '-C',
        'lz4',
        '--paths-from-command',
        'i0fi93@i593.repo.borgbase.com:repo::a1',
        '--',
        'echo',
        TEST_SOURCE_DIR,
    ]


def test_prepare_returns_info_level_when_wifi_disallowed(mocker):
    """Test that prepare() returns level='info' when WiFi is not allowed."""
    default_profile = BackupProfileModel.get()
    mock_monitor = mocker.MagicMock()
    mock_monitor.get_current_wifi.return_value = 'blocked_wifi'
    mocker.patch('vorta.borg.create.get_network_status_monitor', return_value=mock_monitor)
    mocker.patch(
        'vorta.borg.create.WifiSettingModel.select',
        return_value=mocker.MagicMock(where=lambda *a, **kw: mocker.MagicMock(count=lambda: 1)),
    )
    result = BorgCreateJob.prepare(default_profile)
    assert result.get('level') == 'info'


def test_prepare_returns_info_level_when_metered_connection(mocker):
    """Test that prepare() returns level='info' for metered connections."""
    default_profile = BackupProfileModel.get()
    default_profile.dont_run_on_metered_networks = True
    mock_monitor = mocker.MagicMock()
    mock_monitor.get_current_wifi.return_value = None
    mock_monitor.is_network_metered.return_value = True
    mocker.patch('vorta.borg.create.get_network_status_monitor', return_value=mock_monitor)
    result = BorgCreateJob.prepare(default_profile)
    assert result.get('level') == 'info'



from collections import namedtuple

import psutil

import vorta.borg.umount
from vorta.borg.borg_job import BorgJob
from vorta.borg.umount import BorgUmountJob


def test_umount_matches_symlinked_mount_point(monkeypatch):
    """psutil canonicalizes paths (e.g. /home -> /var/home on Silverblue).

    The user-facing mount path stored by Vorta may include a symlink, so
    comparison must use realpath on both sides (#1461).
    """
    user_facing = '/home/test/.mnt/archive'
    canonical = '/var/home/test/.mnt/archive'

    DiskPartitions = namedtuple('DiskPartitions', ['device', 'mountpoint'])

    def fake_disk_partitions(**kwargs):
        return [DiskPartitions('borgfs', canonical)]

    def fake_realpath(p, *args, **kwargs):
        if p == user_facing:
            return canonical
        return p

    monkeypatch.setattr(psutil, 'disk_partitions', fake_disk_partitions)
    monkeypatch.setattr(vorta.borg.umount.os.path, 'realpath', fake_realpath)
    monkeypatch.setattr(BorgJob, 'prepare', classmethod(lambda cls, profile: {'ok': True}))

    ret = BorgUmountJob.prepare(profile=None, mount_point=user_facing)

    assert ret['ok'] is True
    assert ret['mount_point'] == user_facing
    assert canonical in ret['active_mount_points']



from types import SimpleNamespace

import pytest
from PyQt6.QtWidgets import QWidget

from vorta.store.models import BackupProfileModel, RepoModel
from vorta.views.base_tab import BaseTab


class FakeSignal:
    def __init__(self):
        self._slots = []

    def connect(self, slot):
        self._slots.append(slot)
        return slot

    def disconnect(self, slot):
        self._slots.remove(slot)

    def emit(self, *args, **kwargs):
        for slot in list(self._slots):
            try:
                slot(*args, **kwargs)
            except TypeError:
                slot()


class DummyTab(BaseTab, QWidget):
    def __init__(self, parent=None, profile_provider=None):
        super().__init__(parent=parent, profile_provider=profile_provider)
        self.profile_change_count = 0
        self.palette_change_count = 0
        self.backup_finished_count = 0
        self.populate_count = 0

    def on_profile_change(self):
        self.profile_change_count += 1

    def on_palette_change(self):
        self.palette_change_count += 1

    def on_backup_finished(self):
        self.backup_finished_count += 1

    def populate_from_profile(self):
        self.populate_count += 1


def test_base_tab_uses_injected_profile_provider(qapp):
    repo = RepoModel.get(id=1)
    injected_profile = BackupProfileModel.create(name='Injected', repo=repo.id)

    tab = DummyTab(profile_provider=lambda: BackupProfileModel.get(id=injected_profile.id))

    assert tab.profile().id == injected_profile.id

    tab.save_profile_attr('compression', 'none')
    assert BackupProfileModel.get(id=injected_profile.id).compression == 'none'

    tab.save_repo_attr('name', 'Injected Repo')
    assert RepoModel.get(id=repo.id).name == 'Injected Repo'


def test_base_tab_cleans_up_tracked_connections(qapp):
    profile = BackupProfileModel.get(id=1)
    tab = DummyTab(profile_provider=lambda: BackupProfileModel.get(id=profile.id))
    fake_app = SimpleNamespace(
        profile_changed_event=FakeSignal(),
        paletteChanged=FakeSignal(),
        backup_finished_event=FakeSignal(),
    )
    tab.app = fake_app
    tab.track_profile_change(tab.on_profile_change)
    tab.track_palette_change(tab.on_palette_change)
    tab.track_backup_finished(tab.on_backup_finished)

    fake_app.profile_changed_event.emit()
    fake_app.paletteChanged.emit(qapp.palette())
    fake_app.backup_finished_event.emit({})
    assert tab.profile_change_count == 1
    assert tab.palette_change_count == 1
    assert tab.backup_finished_count == 1

    tab._cleanup_tracked_connections()
    fake_app.profile_changed_event.emit()
    fake_app.paletteChanged.emit(qapp.palette())
    fake_app.backup_finished_event.emit({})
    assert tab.profile_change_count == 1
    assert tab.palette_change_count == 1
    assert tab.backup_finished_count == 1


def test_base_tab_track_methods_support_call_now(qapp):
    profile = BackupProfileModel.get(id=1)
    tab = DummyTab(profile_provider=lambda: BackupProfileModel.get(id=profile.id))
    fake_app = SimpleNamespace(
        profile_changed_event=FakeSignal(),
        paletteChanged=FakeSignal(),
        backup_finished_event=FakeSignal(),
    )
    tab.app = fake_app

    tab.track_profile_change(call_now=True)
    tab.track_palette_change(tab.on_palette_change, call_now=True)
    tab.track_backup_finished(tab.on_backup_finished, call_now=True)

    assert tab.populate_count == 1
    assert tab.palette_change_count == 1
    assert tab.backup_finished_count == 1


def test_base_tab_bind_attr_helpers_update_models(qapp):
    repo = RepoModel.get(id=1)
    profile = BackupProfileModel.get(id=1)
    tab = DummyTab(profile_provider=lambda: BackupProfileModel.get(id=profile.id))

    profile_signal = FakeSignal()
    repo_signal = FakeSignal()
    tab.bind_profile_attr(profile_signal, 'compression')
    tab.bind_repo_attr(repo_signal, 'name')

    profile_signal.emit('none')
    repo_signal.emit('Renamed Repo')

    assert BackupProfileModel.get(id=profile.id).compression == 'none'
    assert RepoModel.get(id=repo.id).name == 'Renamed Repo'


def test_base_tab_falls_back_to_window_current_profile(qapp, qtbot):
    profile = BackupProfileModel.get(id=1)
    host = QWidget()
    host.current_profile = profile
    qtbot.addWidget(host)

    tab = DummyTab(parent=host)

    assert tab.current_profile().id == profile.id
    assert tab.profile().id == profile.id


def test_base_tab_handles_missing_current_profile(qapp, qtbot):
    host = QWidget()
    qtbot.addWidget(host)
    tab = DummyTab(parent=host)

    assert tab.current_profile() is None
    with pytest.raises(RuntimeError, match="No active backup profile"):
        tab.profile()
